@@ -1,6 +1,6 @@
-#Copyright ReportLab Europe Ltd. 2000-2012
+#Copyright ReportLab Europe Ltd. 2000-2017
 #see license.txt for license details
-#history http://www.reportlab.co.uk/cgi-bin/viewcvs.cgi/public/reportlab/trunk/reportlab/docs/userguide/ch5_paragraphs.py
+#history https://hg.reportlab.com/hg-public/reportlab/log/tip/docs/userguide/ch5_paragraphs.py
 from tools.docco.rl_doc_utils import *
 
 #begin chapter oon paragraphs
@@ -37,13 +37,14 @@ stylesheet=getSampleStyleSheet()
 normalStyle = stylesheet['Normal']
 """)
 disc("""
-The options which can be set for a $Paragraph$ can be seen from the $ParagraphStyle$ defaults.
+The options which can be set for a $Paragraph$ can be seen from the $ParagraphStyle$ defaults. The values with leading underscore ('_') are derived from the defaults in module $reportlab.rl_config$ which are derived from
+module $reportlab.rl_settings$.
 """)
 heading4("$class ParagraphStyle$")
 eg("""
 class ParagraphStyle(PropertySet):
     defaults = {
-        'fontName':'Times-Roman',
+        'fontName':_baseFontName,
         'fontSize':10,
         'leading':12,
         'leftIndent':0,
@@ -52,7 +53,7 @@ class ParagraphStyle(PropertySet):
         'alignment':TA_LEFT,
         'spaceBefore':0,
         'spaceAfter':0,
-        'bulletFontName':'Times-Roman',
+        'bulletFontName':_baseFontName,
         'bulletFontSize':10,
         'bulletIndent':0,
         'textColor': black,
@@ -67,8 +68,22 @@ class ParagraphStyle(PropertySet):
         'textTransform':None,
         'endDots':None,
         'splitLongWords':1,
-        'underlineProportion': _baseUnderlineProportion,
+        'underlineWidth': _baseUnderlineWidth,
         'bulletAnchor': 'start',
+        'justifyLastLine': 0,
+        'justifyBreaks': 0,
+        'spaceShrinkage': _spaceShrinkage,
+        'strikeWidth': _baseStrikeWidth,    #stroke width
+        'underlineOffset': _baseUnderlineOffset,    #fraction of fontsize to offset underlines
+        'underlineGap': _baseUnderlineGap,      #gap for double/triple underline
+        'strikeOffset': _baseStrikeOffset,  #fraction of fontsize to offset strikethrough
+        'strikeGap': _baseStrikeGap,        #gap for double/triple strike
+        'linkUnderline': _platypus_link_underline,
+        #'underlineColor':  None,
+        #'strikeColor': None,
+        'hyphenationLang': _hyphenationLang,
+        'uriWasteReduce': _uriWasteReduce,
+        'embeddedHyphenation': _embeddedHyphenation,
         }
 """)
 
@@ -170,12 +185,33 @@ The default color of the text can be set with $textColor$ and the paragraph back
 colour can be set with $backColor$. The paragraph's border properties may be changed using
 $borderWidth$, $borderPadding$, $borderColor$ and $borderRadius$.""")
 
-disc("""The $textTransform$ attribute can be <b><i>None</i></b>, <i>'upper'</i> or <i>'lower'</i> to get the obvious result.""")
+disc("""The $textTransform$ attribute can be <b><i>None</i></b>, <i>'uppercase'</i> or <i>'lowercase'</i> to get the obvious result and <i>'capitalize'</i> to get initial letter capitalization.""")
 disc("""Attribute $endDots$ can be <b><i>None</i></b>, a string, or an object with attributes text and optional fontName, fontSize, textColor,  backColor
 and dy(y offset) to specify trailing matter on the last line of left/right justified paragraphs.""")
 disc("""The $splitLongWords$ attribute can be set to a false value to avoid splitting very long words.""")
-disc("""The $underLineProportion$ attribute can be set to a true or false value to control whether underlines are proportional to the font size.""")
 disc("""Attribute $bulletAnchor$ can be <i>'start'</i>, <i>'middle'</i>, <i>'end'</i> or <i>'numeric'</i> to control where the bullet is anchored.""")
+disc("""The $justifyBreaks$ attribute controls whether lines deliberately broken with a $&lt;br/&gt;$ tag should be justified""")
+disc("""Attribute $spaceShrinkage$ is a fractional number specifiying by how much the space of a paragraph
+line may be shrunk in order to make it fit; typically it is something like 0.05""")
+disc("""The $underlineWidth$, $underlineOffset$, $underlineGap$ &amp; $underlineColor$ attributes control the underline behaviour when the $&lt;u&gt;$ or
+a linking tag is used. Those tags can have override values of these attributes. The attribute value  for width &amp; offset
+is a $fraction * Letter$ where letter can be one of $P$, $L$, $f$ or $F$ representing fontSize proportions. $P$ uses the fontsize at the tag, $F$ is the maximum fontSize in the tag, $f$ is the initial fontsize inside the tag.
+$L$ means the global (paragrpah style) font size.
+$strikeWidth$, $strikeOffset$, $strikeGap$ &amp; $strikeColor$ attributes do the same for strikethrough lines.
+""")
+disc("""Attribute $linkUnderline$ controls whether link tags are automatically underlined.""") 
+disc("""If the $pyphen$ python module is installed attribute $hyphenationLang$ controls which language will be used to hyphenate words without explicit embedded hyphens.""")
+disc("""If $embeddedHyphenation$ is set then attempts will be made to split words with embedded hyphens.""")
+disc("""Attribute $uriWasteReduce$ controls how we attempt to split long uri's. It is the fraction of a line that we regard as too much waste. The default in module
+$reportlab.rl_settings$ is <i>0.5</i> which means that we will try and split a word that looks like a uri if we would waste at least half of the line.""")
+disc("""Currently the hyphenation and uri splitting are turned off by default. You need to modify the default settings by using the file $~/.rl_settings$ or adding a module $reportlab_settings.py$ to the python path. Suitable values are""")
+eg("""
+    hyphenationLanguage='en_GB'
+    embeddedHyphenation=1
+    uriWasteReduce=0.3
+    """)
+
+
 
 heading2("Paragraph XML Markup Tags")
 disc("""XML markup can be used to modify or specify the
@@ -202,15 +238,15 @@ def getAttrs(A):
     for k, v in A.items():
         a = v[0]
         if a not in S:
-            S[a] = k
+            S[a] = [k]
         else:
-            S[a] = "%s, %s" %(S[a],k)
+            S[a].append(k)
 
-    K = list(S.keys())
+    K = list(sorted(S.keys()))
     K.sort()
     D=[('Attribute','Synonyms')]
     for k in K:
-        D.append((k,S[k]))
+        D.append((k,", ".join(list(sorted(S[k])))))
     cols=2*[None]
     rows=len(D)*[None]
     return D,cols,rows
@@ -234,7 +270,7 @@ much intra-paragraph markup may not need translating. The
 table below shows the allowed attributes and synonyms in the
 outermost paragraph tag.""")
 
-
+CPage(1)
 heading2("Intra-paragraph markup")
 disc("""<![CDATA[Within each paragraph, we use a basic set of XML tags
 to provide markup.  The most basic of these are bold (<b>...</b>),
@@ -282,15 +318,18 @@ of the peace.  How do you plead?""", "The $font$ tag")
 
 heading3("Superscripts and Subscripts")
 disc("""Superscripts and subscripts are supported with the
-<![CDATA[<super> and <sub> tags, which work exactly
-as you might expect.  In addition, most greek letters
+<![CDATA[<super>/<sup> and <sub> tags, which work exactly
+as you might expect. Additionally these three tags have
+attributes rise and size to optionally set the rise/descent
+and font size for the  superscript/subscript text.
+In addition, most greek letters
 can be accessed by using the <greek></greek>
 tag, or with mathML entity names.]]>""")
 
 ##parabox2("""<greek>epsilon</greek><super><greek>iota</greek>
 ##<greek>pi</greek></super> = -1""", "Greek letters and subscripts")
 
-parabox2("""Equation (&alpha;): <greek>e</greek> <super><greek>ip</greek></super>  = -1""",
+parabox2("""Equation (&alpha;): <greek>e</greek> <super rise=9 size=6><greek>ip</greek></super>  = -1""",
          "Greek letters and superscripts")
 
 heading3("Inline Images")
@@ -305,6 +344,19 @@ This &lt;img/&gt; <img src="../images/testimg.gif" valign="-4"/> is aligned <b>-
 This &lt;img/&gt; <img src="../images/testimg.gif" valign="+4"/> is aligned <b>+4</b>.<br/><br/>
 This &lt;img/&gt; <img src="../images/testimg.gif" width="10"/> has width <b>10</b>.<br/><br/>
 </para>""","Inline images")
+disc("""The $src$ attribute can refer to a remote location eg $src="https://www.reportlab.com/images/logo.gif"$. By default we set $rl_config.trustedShemes$ to $['https','http', 'file', 'data', 'ftp']$ and
+$rl_config.trustedHosts=None$ the latter meaning no-restriction. You can modify these variables using one of the override files eg $reportlab_settings.py$ or $~/.reportlab_settings$. Or as comma seprated strings in the 
+environment variables $RL_trustedSchemes$ &amp; $RL_trustedHosts$. Note that the $trustedHosts$ values may contain <b>glob</b> wild cars so <i>*.reportlab.com</i> will match the obvious domains.
+<br/><span color="red"><b>*NB*</b></span> use of <i>trustedHosts</i> and or <i>trustedSchemes</i> may not control behaviour &amp; actions when $URI$ patterns
+are detected by the viewer application.""")
+
+heading3("The $&lt;u&gt;$ &amp; $&lt;strike&gt;$ tags")
+disc("""These tags can be used to carry out explicit underlineing or strikethroughs. These tags have
+attributes $width$, $offset$, $color$, $gap$ &amp; $kind$. The $kind$ attribute controls how many
+lines will be drawn (default $kind=1$) and when $kind>1$ the $gap$ attribute controls the disatnce between lines.""")
+
+heading3("The $&lt;nobr&gt;$ tag")
+disc("""If hyphenation is in operation the $&lt;nobr&gt;$ tag suppresses it so $&lt;nobr&gt;averylongwordthatwontbebroken&lt;/nobr&gt;$ won't be broken.""")
 
 heading3("Numbering Paragraphs and Lists")
 disc("""The $&lt;seq&gt;$ tag provides comprehensive support
@@ -382,7 +434,7 @@ disc("""The &lt;bullet&gt; tag is only allowed once in a given paragraph and its
 overrides the implied bullet style and ^bulletText^ specified in the  ^Paragraph^
 creation.
 """)
-parabox("""<bullet>\u2022</bullet>this is a bullet point.  Spam
+parabox("""<bullet>&bull;</bullet>this is a bullet point.  Spam
 spam spam spam spam spam spam spam spam spam spam spam
 spam spam spam spam spam spam spam spam spam spam """,
         styleSheet['Bullet'],
